@@ -3,20 +3,19 @@ pub mod renderable;
 pub mod title;
 
 use crate::ansi::CursorShape;
-use crate::context::grid::ContextDimension;
-use crate::context::grid::ContextGrid;
-use crate::context::grid::Delta;
+use crate::context::grid::{ContextDimension, ContextGrid, ContextGridItem, Delta};
 use crate::context::title::{
     create_title_extra_from_context, update_title, ContextManagerTitles,
 };
 use crate::event::sync::FairMutex;
-use crate::event::RioEvent;
+use crate::event::{Msg, RioEvent};
 use crate::ime::Ime;
 use crate::messenger::Messenger;
-use crate::performer::Machine;
+use crate::performer::{self, Machine};
 use renderable::Cursor;
 use renderable::RenderableContent;
 use rio_backend::config::Shell;
+use smallvec::{smallvec, SmallVec};
 
 use rio_backend::crosswords::{Crosswords, MIN_COLUMNS, MIN_LINES};
 use rio_backend::error::{RioError, RioErrorLevel, RioErrorType};
@@ -26,16 +25,19 @@ use rio_backend::selection::SelectionRange;
 use rio_backend::sugarloaf::{font::SugarloafFont, Object, SugarloafErrors};
 use std::borrow::Cow;
 use std::error::Error;
+use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::Arc;
+use std::thread::JoinHandle;
 use std::time::{Duration, Instant};
 
+// Global atomic counter for generating unique route IDs
+static ROUTE_ID_COUNTER: AtomicUsize = AtomicUsize::new(1);
+
 #[cfg(target_os = "windows")]
 use teletypewriter::create_pty;
 #[cfg(not(target_os = "windows"))]
 use teletypewriter::{create_pty_with_fork, create_pty_with_spawn};
 
-const DEFAULT_CONTEXT_CAPACITY: usize = 28;
-
 pub struct Context<T: EventListener> {
     pub route_id: usize,
     pub terminal: Arc<FairMutex<Crosswords<T>>>,
@@ -48,10 +50,14 @@ pub struct Context<T: EventListener> {
     pub rich_text_id: usize,
     pub dimension: ContextDimension,
     pub ime: Ime,
+    _io_thread: Option<JoinHandle<(Machine<teletypewriter::Pty, T>, performer::State)>>,
 }
 
 impl<T: rio_backend::event::EventListener> Drop for Context<T> {
     fn drop(&mut self) {
+        // Shutdown the terminal's PTY.
+        let _ = self.messenger.channel.send(Msg::Shutdown);
+
         #[cfg(not(target_os = "windows"))]
         teletypewriter::kill_pid(self.shell_pid as i32);
     }
@@ -66,6 +72,12 @@ impl<T: EventListener> Context<T> {
             let mut terminal = self.terminal.lock();
             let display_offset = terminal.display_offset();
             terminal.update_selection_damage(selection_range, display_offset);
+            drop(terminal);
+
+            // Mark pending update as dirty so the selection change is rendered
+            self.renderable_content
+                .pending_update
+                .invalidate_full(&self.terminal);
         }
 
         self.renderable_content.selection_range = selection_range;
@@ -74,8 +86,9 @@ impl<T: EventListener> Context<T> {
     #[inline]
     pub fn set_hyperlink_range(&mut self, hyperlink_range: Option<SelectionRange>) {
         self.renderable_content.hyperlink_range = hyperlink_range;
-        self.renderable_content.has_pending_updates =
-            Some(rio_backend::event::TerminalDamage::Full);
+        self.renderable_content
+            .pending_update
+            .invalidate_full(&self.terminal);
     }
 
     #[inline]
@@ -109,11 +122,12 @@ pub struct ContextManagerConfig {
     pub keyboard: rio_backend::config::keyboard::Keyboard,
 }
 
+const DEFAULT_CONTEXT_CAPACITY: usize = 28;
+
 pub struct ContextManager<T: EventListener> {
-    contexts: Vec<ContextGrid<T>>,
+    contexts: SmallVec<[ContextGrid<T>; DEFAULT_CONTEXT_CAPACITY]>,
     current_index: usize,
     current_route: usize,
-    acc_current_route: usize,
     #[allow(unused)]
     capacity: usize,
     event_proxy: T,
@@ -151,6 +165,7 @@ pub fn create_dead_context<T: rio_backend::event::EventListener>(
         rich_text_id,
         dimension,
         ime: Ime::new(),
+        _io_thread: None,
     }
 }
 
@@ -160,7 +175,6 @@ pub fn create_mock_context<
 >(
     event_proxy: T,
     window_id: WindowId,
-    route_id: usize,
     rich_text_id: usize,
     dimension: ContextDimension,
 ) -> Context<T> {
@@ -182,7 +196,6 @@ pub fn create_mock_context<
         (&Cursor::default(), false),
         event_proxy.clone(),
         window_id,
-        route_id,
         rich_text_id,
         dimension,
         &config,
@@ -196,11 +209,11 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         cursor_state: (&Cursor, bool),
         event_proxy: T,
         window_id: WindowId,
-        route_id: usize,
         rich_text_id: usize,
         dimension: ContextDimension,
         config: &ContextManagerConfig,
     ) -> Result<Context<T>, Box<dyn Error>> {
+        let route_id = ROUTE_ID_COUNTER.fetch_add(1, Ordering::SeqCst);
         let cols: u16 = dimension.columns.try_into().unwrap_or(MIN_COLUMNS as u16);
         let rows: u16 = dimension.lines.try_into().unwrap_or(MIN_LINES as u16);
 
@@ -270,17 +283,14 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             }
         }
 
-        let machine = Machine::new(
-            Arc::clone(&terminal),
-            pty,
-            event_proxy.clone(),
-            window_id,
-            route_id,
-        )?;
+        let machine =
+            Machine::new(Arc::clone(&terminal), pty, event_proxy.clone(), window_id)?;
         let channel = machine.channel();
-        if config.spawn_performer {
-            machine.spawn();
-        }
+        let io_thread = if config.spawn_performer {
+            Some(machine.spawn())
+        } else {
+            None
+        };
 
         let messenger = Messenger::new(channel);
 
@@ -296,6 +306,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             renderable_content: RenderableContent::new(cursor_state.0.clone()),
             dimension,
             ime: Ime::new(),
+            _io_thread: io_thread,
         })
     }
 
@@ -316,7 +327,6 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             cursor_state,
             event_proxy.clone(),
             window_id,
-            route_id,
             rich_text_id,
             size,
             &ctx_config,
@@ -365,8 +375,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         Ok(ContextManager {
             current_index: 0,
             current_route: 0,
-            acc_current_route: 0,
-            contexts: vec![ContextGrid::new(
+            contexts: smallvec![ContextGrid::new(
                 initial_context,
                 margin,
                 ctx_config.split_color,
@@ -404,7 +413,6 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             event_proxy.clone(),
             window_id,
             0,
-            0,
             ContextDimension::default(),
             &config,
         )?;
@@ -414,8 +422,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         Ok(ContextManager {
             current_index: 0,
             current_route: 0,
-            acc_current_route: 0,
-            contexts: vec![ContextGrid::new(
+            contexts: smallvec![ContextGrid::new(
                 initial_context,
                 Delta::<f32>::default(),
                 config.split_color,
@@ -699,7 +706,14 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
     }
 
     #[inline]
-    pub fn contexts_mut(&mut self) -> &mut Vec<ContextGrid<T>> {
+    pub fn get_mut(&mut self, route_id: usize) -> Option<&mut ContextGridItem<T>> {
+        self.contexts[self.current_index].get_mut(route_id)
+    }
+
+    #[inline]
+    pub fn contexts_mut(
+        &mut self,
+    ) -> &mut SmallVec<[ContextGrid<T>; DEFAULT_CONTEXT_CAPACITY]> {
         &mut self.contexts
     }
 
@@ -875,7 +889,6 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             cloned_config.working_dir = working_dir;
         }
 
-        self.acc_current_route += 1;
         let current = self.current();
         let cursor = current.cursor_from_ref();
 
@@ -883,19 +896,19 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             (&cursor, current.renderable_content.has_blinking_enabled),
             self.event_proxy.clone(),
             self.window_id,
-            self.acc_current_route,
             rich_text_id,
             self.current().dimension,
             &cloned_config,
         ) {
             Ok(new_context) => {
+                let new_route_id = new_context.route_id;
                 if split_down {
                     self.contexts[self.current_index].split_down(new_context);
                 } else {
                     self.contexts[self.current_index].split_right(new_context);
                 }
 
-                self.current_route = self.acc_current_route;
+                self.current_route = new_route_id;
             }
             Err(..) => {
                 tracing::error!("not able to create a new context");
@@ -932,7 +945,6 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             keyboard: config.keyboard,
         };
 
-        self.acc_current_route += 1;
         let current = self.current();
         let cursor = current.cursor_from_ref();
 
@@ -940,19 +952,19 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             (&cursor, current.renderable_content.has_blinking_enabled),
             self.event_proxy.clone(),
             self.window_id,
-            self.acc_current_route,
             rich_text_id,
             self.current().dimension,
             &context_manager_config,
         ) {
             Ok(new_context) => {
+                let new_route_id = new_context.route_id;
                 if split_down {
                     self.contexts[self.current_index].split_down(new_context);
                 } else {
                     self.contexts[self.current_index].split_right(new_context);
                 }
 
-                self.current_route = self.acc_current_route;
+                self.current_route = new_route_id;
             }
             Err(..) => {
                 tracing::error!("not able to create a new context");
@@ -1000,7 +1012,6 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
                 cloned_config.working_dir = working_dir;
             }
 
-            self.acc_current_route += 1;
             let current = self.current();
             let cursor = current.cursor_from_ref();
             let mut dimension = current.dimension;
@@ -1014,7 +1025,6 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
                 (&cursor, current.renderable_content.has_blinking_enabled),
                 self.event_proxy.clone(),
                 self.window_id,
-                self.acc_current_route,
                 rich_text_id,
                 dimension,
                 &cloned_config,
