@@ -15,6 +15,16 @@ pub fn default_cursor_interval() -> u64 {
     800
 }
 
+#[inline]
+pub fn default_title_placeholder() -> Option<String> {
+    Some(String::from("▲"))
+}
+
+#[inline]
+pub fn default_title_content() -> String {
+    String::from("{{ TITLE || PROGRAM }}")
+}
+
 #[inline]
 pub fn default_padding_y() -> [f32; 2] {
     [0., 0.]
@@ -64,7 +74,7 @@ pub fn default_opacity() -> f32 {
 
 #[inline]
 pub fn default_option_as_alt() -> String {
-    String::from("None")
+    String::from("none")
 }
 
 #[inline]
@@ -108,7 +118,7 @@ pub fn default_window_width() -> i32 {
 
 #[inline]
 pub fn default_window_height() -> i32 {
-    500
+    490
 }
 
 #[inline]
@@ -124,6 +134,11 @@ pub fn default_disable_ctlseqs_alt() -> bool {
     }
 }
 
+#[inline]
+pub fn default_ime_cursor_positioning() -> bool {
+    true
+}
+
 pub fn default_config_file_content() -> String {
     r#"
 # Hide the cursor while typing
@@ -142,7 +157,8 @@ pub fn default_config_file_content() -> String {
 # Theme
 #
 # It makes Rio look for the specified theme in the themes folder
-# (macos and linux: ~/.config/rio/themes/dracula.toml)
+# (macos: ~/.config/rio/themes/dracula.toml)
+# (linux: $XDG_HOME_CONFIG/rio/themes/dracula.toml or ~/.config/rio/themes/dracula.toml)
 # (windows: C:\Users\USER\AppData\Local\rio\themes\dracula.toml)
 #
 # Example:
@@ -157,7 +173,7 @@ pub fn default_config_file_content() -> String {
 
 # Padding-y
 #
-# define y axis padding based on a format [top, left]
+# define y axis padding based on a format [top, bottom]
 # (default is [0, 0])
 #
 # Example:
@@ -252,6 +268,11 @@ pub fn default_config_file_content() -> String {
 #
 # • decorations - Set window decorations, options: "Enabled", "Disabled", "Transparent", "Buttonless"
 #
+# • colorspace - Set the color space for the window
+#     - "srgb" (default on non-macOS)
+#     - "display-p3" (default on macOS)
+#     - "rec2020"
+#
 # Example:
 # [window]
 # width = 600
@@ -260,6 +281,7 @@ pub fn default_config_file_content() -> String {
 # opacity = 1.0
 # blur = false
 # decorations = "enabled"
+# colorspace = "display-p3"
 
 # Renderer
 #
@@ -299,10 +321,15 @@ pub fn default_config_file_content() -> String {
 # disable-ctlseqs-alt - Disable ctlseqs with ALT keys
 #   - For example: Terminal.app does not deal with ctlseqs with ALT keys
 #
+# ime-cursor-positioning - Enable IME cursor positioning
+#   - When enabled, the IME input popup will appear at the cursor position
+#   - Default is true
+#
 # Example:
 # [keyboard]
 # use-kitty-keyboard-protocol = false
 # disable-ctlseqs-alt = false
+# ime-cursor-positioning = true
 
 # Fonts
 #
@@ -330,28 +357,34 @@ pub fn default_config_file_content() -> String {
 # [fonts]
 # hinting = false
 #
-# Example:
+# You can also map the specified Unicode codepoints to a particular font.
+# [fonts]
+# symbol-map = [
+#   { start = "2297", end = "2299", font-family = "Cascadia Code NF" }
+# ]
+#
+# Simple example:
 # [fonts]
 # size = 18
 #
 # [fonts.regular]
 # family = "cascadiamono"
-# style = "normal"
+# style = "Normal"
 # weight = 400
 #
 # [fonts.bold]
 # family = "cascadiamono"
-# style = "normal"
+# style = "Normal"
 # weight = 800
 #
 # [fonts.italic]
 # family = "cascadiamono"
-# style = "italic"
+# style = "Italic"
 # weight = 400
 #
 # [fonts.bold-italic]
 # family = "cascadiamono"
-# style = "italic"
+# style = "Italic"
 # weight = 800
 
 # Scroll
@@ -446,7 +479,7 @@ pub fn default_config_file_content() -> String {
 # Bindings
 #
 # Create custom Key bindings for Rio terminal
-# More information in: https://raphamorim.io/rio/docs/config/bindings
+# More information in: https://raphamorim.io/rio/docs/key-bindings
 #
 # Example:
 # [bindings]
